// DesktopArray.cpp
// implementation file for the class DesktopArray

#include "stdafx.h"

#include<winnls.h>
#include<string.h>

#include "resource.h"

#include "DeskArr.h"

desktop_entry::~desktop_entry() {
	Shrink();
}

void desktop_entry::Shrink() {
	m_string.Empty();
	m_attribs = 0;
	if (m_pcontextmenu) {
		m_pcontextmenu->Release();
		m_pcontextmenu = NULL;
	}
	if (m_pmalloc) {					// need OLE malloc ptr to do this!
		if ( m_pidl) {
			m_pmalloc->Free(m_pidl);
			m_pidl = NULL;
		}
	}
}

DesktopArray::DesktopArray()
: m_nbr_items(0),m_must_reload(1),m_pmalloc(NULL),m_hwnd(NULL)
{
	// get the shell memory allocator
	if ( NOERROR != SHGetMalloc(&m_pmalloc) ) m_pmalloc = NULL;
	else {	// got the allocator; can now init all m_arr members
		int idx;
		for (idx = 0; idx < kMaxEntries ; idx++ ) {
			m_arr[idx].m_pmalloc = m_pmalloc;
		}
	}
	m_invoker_array.SetSize(0,100);
}

DesktopArray::~DesktopArray() {
	if ( m_pmalloc ) {
		m_pmalloc->Release();
		m_pmalloc = NULL;
	}
}

void DesktopArray::Shrink() {
	int idx;
	for (idx = 0; idx < kMaxEntries ; idx++ ) {
		m_arr[idx].Shrink();
	}
	m_invoker_array.RemoveAll();
}

class stringable_strret {
public:
	operator LPSTRRET() { return &m_data; }
	operator LPCSTR();
	void SetPidl(LPITEMIDLIST pidl) { m_pidl = pidl; }
	void SetMalloc(LPMALLOC pmalloc) { m_pmalloc = pmalloc; }
	void Cleanup();
private:
	STRRET m_data;
	char extra_space[100];
	LPITEMIDLIST m_pidl;
	LPMALLOC m_pmalloc;
};

stringable_strret::operator LPCSTR() {
	switch ( m_data.uType ) {
	case STRRET_WSTR:
		{
			CString temp(m_data.pOleStr);
			wsprintf(m_data.cStr,"%s",(LPCSTR)temp);
			Cleanup();
			m_data.uType = STRRET_CSTR;
			return (LPCSTR)m_data.cStr;
		}
	case STRRET_CSTR:
		return (LPCSTR)m_data.cStr;
	case STRRET_OFFSET:
		return &((LPCSTR)m_pidl)[m_data.uOffset];
	}
	return NULL;
}

void stringable_strret::Cleanup() {
	if ( ( m_data.uType == STRRET_WSTR ) && ( m_data.pOleStr ) ) {
		// must free the OLE string supplied
		m_pmalloc->Free(m_data.pOleStr);
		m_data.pOleStr = NULL;
	}
}

void DesktopArray::FillFromDesktop(){

	UINT cmd_id = IDC_DESKTOP;
	SHCONTF what2show = (SHCONTF) (SHCONTF_FOLDERS | SHCONTF_NONFOLDERS /*| SHCONTF_INCLUDEHIDDEN*/ );
	LPSHELLFOLDER pfolder = NULL;
	LPENUMIDLIST penum = NULL;
	LPITEMIDLIST pidl = NULL;
	stringable_strret str;
	int this_index = 0, this_one_ok;
	ULONG attribs;
#ifdef _DEBUG
	char buf[MAX_PATH];
#endif // _DEBUG

	if (m_nbr_items && !m_must_reload) goto get_out;
	m_must_reload = 1;


	if (!m_hwnd) {
		// need to get the main window handle
		CWinApp* p = AfxGetApp();
		if (p && p->m_pMainWnd) {
			m_hwnd = p->m_pMainWnd->GetSafeHwnd();
		}
	}
	ASSERT(m_hwnd);
	if (!m_hwnd) goto get_out;			// need m_hwnd

	if (!m_pmalloc) goto get_out;
	Shrink();

	str.SetMalloc(m_pmalloc); // preinitialize; it's safe to do now
	
	if ( NOERROR != SHGetDesktopFolder(&pfolder) ) goto get_out;

	if ( NOERROR != pfolder->EnumObjects(m_hwnd,what2show,&penum) ) goto get_out;

	while ( NOERROR == penum->Next(1,&pidl,NULL) ) {
		if ( pidl ) {

			this_one_ok = 1;

#ifdef _DEBUG
			if ( NOERROR == pfolder->GetDisplayNameOf(pidl,SHGDN_FORPARSING,str) ) {
#else // not _DEBUG
			if ( NOERROR == pfolder->GetDisplayNameOf(pidl,SHGDN_NORMAL,str) ) {
#endif // _DEBUG

				UINT utype = ((LPSTRRET)str)->uType;

				str.SetPidl(pidl); // can change with each Next() call above
				CString dn((LPCSTR)str);

#ifdef _DEBUG

				dn += "( ";
				attribs = SFGAO_LINK | SFGAO_FOLDER | SFGAO_FILESYSTEM ;
				if ( NOERROR == pfolder->GetAttributesOf(1,(LPCITEMIDLIST *)&pidl,&attribs) ) {
					if ( attribs & SFGAO_LINK ) dn += "Link ";
					if ( attribs & SFGAO_FOLDER ) dn += "Folder ";
					if ( attribs & SFGAO_FILESYSTEM ) dn += "Filesystem ";
					m_arr[this_index].m_attribs = attribs;
					if (SHGetPathFromIDList(pidl,buf)) { dn += buf; dn+= " "; }
					else dn += "?? ";
				}
				else this_one_ok = 0;
				switch (utype) {
				case STRRET_WSTR: dn += "WSTR "; break;
				case STRRET_OFFSET: dn += "OFFSET "; break;
				case STRRET_CSTR: dn+= "CSTR "; break;
				}
				dn += ")";
#else // _DEBUG
				attribs = SFGAO_FILESYSTEM ; // later we'll want to sort the 'special' items to the top
				if ( NOERROR == pfolder->GetAttributesOf(1,(LPCITEMIDLIST *)&pidl,&attribs) ) {
					m_arr[this_index].m_attribs = attribs;
				}
				else this_one_ok = 0;

#endif // _DEBUG
				m_arr[this_index].m_string = dn;

				// IID_IContextMenu

				str.Cleanup();
			}
			else this_one_ok = 0;

			if (this_one_ok) {
				if ( NOERROR != pfolder->GetUIObjectOf(m_hwnd,
					1,
					(const struct _ITEMIDLIST**)&pidl,
					IID_IContextMenu,
					NULL,
					(void**)&m_arr[this_index].m_pcontextmenu) ) {
						m_arr[this_index].m_pcontextmenu = NULL;
						this_one_ok = 0;
				}
				else {
					m_arr[this_index].m_pidl = pidl;
					pidl = NULL;
				}
			}

			if ( this_one_ok ) {
				this_index++;
				if ( this_index == kMaxEntries ) break;
			}
			if (pidl) {
				m_pmalloc->Free(pidl);
				pidl = NULL;
			}
		}
	}
	m_nbr_items = this_index;

//	IShellFolder

	// now we have the IShellFolder interface


get_out:
	if ( pidl ) {
		m_pmalloc->Free(pidl);
		pidl = NULL;
	}
	if ( penum ) penum->Release();
	if ( pfolder ) pfolder->Release();
	return;
}

// set by DesktopArray::AddAllItemsToMenu(), used by sortfunc()
static desktop_entry* entryarr = NULL;

static int sortfunc(const void* e1,const void* e2) {
	int idx1 = *(int*)e1;
	int idx2 = *(int*)e2;
	int special_diff = !!(entryarr[idx1].m_attribs&SFGAO_FILESYSTEM) - !!(entryarr[idx2].m_attribs&SFGAO_FILESYSTEM);
	if ( special_diff ) return special_diff; // this separates the non-filesystem entries to the top of the sort order
	return stricmp((const char*)entryarr[idx1].m_string,(const char*)entryarr[idx2].m_string);
}

enum menusep_status {
	kMssNothing,
	kMssSpecialOnly,
	kMssFirstNonSpecial,
	kMssAllDone
};

void DesktopArray::AddAllItemsToMenu(CMenu* menu) {
	ASSERT(m_nbr_items>0);
	if (m_nbr_items<0) return;
	int* sortarr = new int[m_nbr_items];
	ASSERT(sortarr);
	if (!sortarr) return;
	menusep_status mss = kMssNothing;
	{									// now we'll sort 'em in alpha order
		int x;
		for ( x=0; x<m_nbr_items; x++ ) {
			sortarr[x] = x;
		}
		ASSERT(entryarr==NULL);
		entryarr = &m_arr[0];
		qsort((void*)sortarr,m_nbr_items,sizeof(int),sortfunc);
		entryarr = NULL;
	}
	int idx = 0;
	while ( idx < m_nbr_items) {
		CMenu m;
		BOOL ret;
		ret = m.CreatePopupMenu();
		ASSERT(ret);
		if (ret) {
			HRESULT rr = m_arr[sortarr[idx]].m_pcontextmenu->QueryContextMenu(
				m.GetSafeHmenu(),	// menu to add items to
				0,					// where in the menu to place items
				0,					// idCmdFirst, first OK identifier
				0x7FFF,				// idCmdLast, last OK identifier
				CMF_NORMAL);		// which items to add
			if (rr > 0) {			// got the menu items
				{
					MENUITEMINFO mii;
					mii.cbSize = sizeof(MENUITEMINFO);
					mii.fMask = MIIM_ID;
					mii.fType = 0;
					mii.fState = 0;
					//mii.wID = 0;						// this is the one we'll set each time
					mii.hSubMenu = NULL;
					mii.hbmpChecked = NULL;
					mii.hbmpUnchecked = NULL;
					mii.dwItemData = 0;
					mii.dwTypeData = 0;
					mii.cch = 0;
					ProcessMenuIntoInvoker(m_arr[sortarr[idx]].m_pcontextmenu,&m,&mii);
					int is_special = 0;
					if ( ! ( m_arr[sortarr[idx]].m_attribs & SFGAO_FILESYSTEM ) ) is_special = 1;
					switch (mss) {
					case kMssNothing:			mss = is_special ? kMssSpecialOnly : kMssAllDone;			break;
					case kMssSpecialOnly:		mss = is_special ? kMssSpecialOnly : kMssFirstNonSpecial;	break;
					case kMssFirstNonSpecial:	mss = kMssAllDone;											break;
					case kMssAllDone:																		break;
					}
				}

				if ( mss==kMssFirstNonSpecial ) menu->InsertMenu( ID_APP_ABOUT, MF_BYCOMMAND|MF_SEPARATOR );

				menu->InsertMenu(ID_APP_ABOUT,MF_BYCOMMAND|MF_POPUP,(unsigned int)m.GetSafeHmenu(),m_arr[sortarr[idx]].m_string);
				m.Detach();
			}
		}
		idx++;
	}
	menu->InsertMenu(ID_APP_ABOUT,MF_BYCOMMAND|MF_SEPARATOR); // separator
	delete[] sortarr;
}

void DesktopArray::ProcessMenuIntoInvoker(LPCONTEXTMENU cxm, CMenu* cmp,LPMENUITEMINFO lpmii) {
	UINT menu_size = cmp->GetMenuItemCount();
	int last_was_separator = 1;
	ASSERT(menu_size!=(UINT)-1);
	if (menu_size==(UINT)-1) return;
	while (menu_size) {
		menu_size--;	// zero-based indexes so decrement here
		UINT id = cmp->GetMenuItemID(menu_size);
		switch (id) { // id is the identifier for the menu item
		case 0:				// separator, ignore it
		case 0xFFFF:		// not docmented, but separator
			if (last_was_separator) cmp->DeleteMenu(menu_size,MF_BYPOSITION);
			last_was_separator = 1;
			break;
		case (UINT)-1:		// it's a pop-up menu
			{
				CMenu* newcmp = cmp->GetSubMenu(menu_size);
				ASSERT(newcmp);
				if (newcmp) {	// got a submenu, process it now
					char* etarget = "Empty";
					char buf[50];
					/*
					Now we check to see if it's an unusable Send To command
					if so we'll delete it
					*/
					if (		(newcmp->GetMenuItemCount() == 1) &&
								( newcmp->GetMenuString(0,buf,sizeof(buf),MF_BYPOSITION) ) &&
								( !strcmp(buf,etarget) ) // the popup is empty
							) {
						cmp->DeleteMenu(menu_size,MF_BYPOSITION);
					}
					else {
						ProcessMenuIntoInvoker(cxm,newcmp,lpmii);
						last_was_separator = 0;
					}
				}
			}
			break;
		default:			// it's a normal menu item so we'll renumber it
			{
				char buf[50];
				if (		( cmp->GetMenuString(menu_size,buf,sizeof(buf),MF_BYPOSITION) ) &&
							  ( ( !strcmp(buf,"Create &Shortcut") ||
							  (!strcmp(buf,"Cu&t")) ||
							  (!strcmp(buf,"&Copy")) ) )
						) {
					cmp->DeleteMenu(menu_size,MF_BYPOSITION);
				}
				else {
					int idx;
					invoker iv;
					iv.cxm = cxm;
					iv.id = id;
					iv.enabled = (cmp->GetMenuState(menu_size,MF_BYPOSITION)&(MF_DISABLED|MF_GRAYED))?FALSE:TRUE;
					idx = m_invoker_array.Add(iv);
					// now renumber the menu item
					lpmii->wID = idx + IDC_DESKTOP;
					::SetMenuItemInfo(cmp->GetSafeHmenu(),menu_size,TRUE,lpmii);
					last_was_separator = 0;
				}
			}
		}
	}
}

int DesktopArray::ExecuteCommandID(UINT id) {
	int retval = 0,idx = id-IDC_DESKTOP;
	if ( idx >= m_invoker_array.GetSize() ) {
		goto get_out;
	}
	ASSERT(m_invoker_array[idx].cxm);
	if (!m_invoker_array[idx].cxm) goto get_out;
	{				// now actually do it
		HRESULT hr;
		CMINVOKECOMMANDINFO cc;
		cc.cbSize = sizeof(cc);
		cc.fMask = 0;
		cc.hwnd = /*m_hwnd;*/ ::GetDesktopWindow();
		cc.lpVerb = MAKEINTRESOURCE(m_invoker_array[idx].id);
		cc.lpParameters = NULL;
		cc.lpDirectory = NULL;
		cc.nShow = SW_SHOWNORMAL;
		cc.dwHotKey = 0;
		cc.hIcon = NULL;
		hr = m_invoker_array[idx].cxm->InvokeCommand(&cc);
		if ( NOERROR == hr )
			retval = 1;
	}
get_out:
	return retval;
}

BOOL DesktopArray::IsCommandIDEnabled(UINT id) {
	BOOL retval = FALSE;
	int idx = id-IDC_DESKTOP;
	if ( idx >= m_invoker_array.GetSize() ) {
		goto get_out;
	}
	ASSERT(m_invoker_array[idx].cxm);
	if (!m_invoker_array[idx].cxm) goto get_out;
	retval = m_invoker_array[idx].enabled;
get_out:
	return retval;
}
