// STI.cpp
// implementation file for the class SystemTrayIcon

#include "stdafx.h"

#include "STI.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CString SystemTrayIcon::m_class;

SystemTrayIcon::SystemTrayIcon()
	: m_ActivateWnd(NULL),
	m_NotifyTargetWnd(NULL),
	m_Menu(NULL),
	m_Enabled(0)
{
	memset(&m_Nid,0,sizeof(NOTIFYICONDATA));
	m_Nid.cbSize = sizeof(NOTIFYICONDATA);
	m_Nid.uID = (UINT)this;				// not really used
	m_Nid.uCallbackMessage = WM_SYSTEM_TRAY_NOTIFY;
}

SystemTrayIcon::~SystemTrayIcon() {
	/*
		remove icon if displayed
		kill external window if exists
	*/
	Enable(FALSE);
	if (m_NotifyTargetWnd) m_NotifyTargetWnd->SendMessage(WM_CLOSE);
}

void SystemTrayIcon::SetIcon(UINT iconid) {
	HICON hic = AfxGetApp()->LoadIcon(iconid);
	internalSetIcon(hic);
}

void SystemTrayIcon::SetIcon(LPCTSTR iconresourcename) {
	HICON hic = AfxGetApp()->LoadIcon(iconresourcename);
	internalSetIcon(hic);
}

void SystemTrayIcon::SetStandardIcon(LPCTSTR iconname) {
	HICON hic = AfxGetApp()->LoadStandardIcon(iconname);
	internalSetIcon(hic);
}

void SystemTrayIcon::SetToolTip(LPCTSTR tip) {
	int tip_was_NULL = m_Nid.szTip[0] ? 0 : 1;
	int new_tip_is_NULL = (tip && tip[0]) ? 0 : 1;

	// don't bother replacing one blank with another
	if (! (tip_was_NULL && new_tip_is_NULL) ) { // OK there's data

		// first replace the text
		int bufsiz = sizeof(m_Nid.szTip) / sizeof(m_Nid.szTip[0]);
		int len = lstrlen(tip);
		if (len>(bufsiz-1)) len = bufsiz-1; // stay in bounds
		memcpy(m_Nid.szTip,tip,len);	// copy the text
		m_Nid.szTip[len] = 0;			// NULL terminate

		// now change the current text if we're active
		if (m_Enabled) {
			m_Nid.uFlags = NIF_TIP;
			VERIFY(Shell_NotifyIcon(NIM_MODIFY,&m_Nid));
		}
	}
}

int SystemTrayIcon::Enable(BOOL bOn) {
	int retval = IsEnabled();
	DWORD msg;
	BOOL success;
	if ( !!bOn == !!m_Enabled ) return retval; // already in desired state
	if (bOn) {
		if (!m_NotifyTargetWnd) {

			// register a class name if necessary
			if (m_class.IsEmpty()) {
				UINT style = CS_NOCLOSE;
				m_class = AfxRegisterWndClass(style);
				if (m_class.IsEmpty()) return retval;	// failed to register
			}

			m_NotifyTargetWnd = new CNotifyTargetWnd(this);
			ASSERT(m_NotifyTargetWnd);
			if (!m_NotifyTargetWnd) return retval;		// failed to allocate

			success=m_NotifyTargetWnd->CreateEx(
				WS_EX_NOPARENTNOTIFY,	// ext style
				m_class,				// class name
				NULL,					// window name
				WS_OVERLAPPEDWINDOW,	// style
				0,0,					// x and y
				50,50,					// width and height totally arbitrary
				NULL,					// parent not wanted
				NULL,					// no menu
				NULL);					// no params

			ASSERT(success);
			if (!success) {
				delete m_NotifyTargetWnd;
				m_NotifyTargetWnd = NULL;
			}
			m_Nid.hWnd = m_NotifyTargetWnd->m_hWnd;
		}
		ASSERT(m_NotifyTargetWnd);
		ASSERT(m_Nid.hWnd);
		ASSERT(m_Nid.hIcon);
		ASSERT(m_Nid.uCallbackMessage);
		if (!m_NotifyTargetWnd || !m_Nid.hIcon) return retval; // can't proceed

		msg = NIM_ADD;					// adding icon
		m_Nid.uFlags = NIF_ICON | NIF_MESSAGE;
		if ( m_Nid.szTip[0] ) m_Nid.uFlags |= NIF_TIP; // tip?
	}
	else {
		msg = NIM_DELETE;				// deleting icon
		m_Nid.uFlags = 0;
	}
	success = Shell_NotifyIcon(msg,&m_Nid);
	ASSERT(success);
	if (success) m_Enabled = bOn ? 1 : 0;
	return retval;
}

void SystemTrayIcon::OnRButtonUp() {
	CMenu menu;
	BOOL success;

	ASSERT(m_ActivateWnd);				// check window that will own menu
	if (!m_ActivateWnd) return;

	ASSERT(m_Menu);						// check menu identifier
	if (!m_Menu) return;

	success = menu.LoadMenu(m_Menu);	// load the menu
	ASSERT(success);					// check success of menu load
	if (!success) return;

	CMenu* submenu = menu.GetSubMenu(0); // extract submenu
	ASSERT(submenu);					// check success of submenu
	if (!submenu) return;

	// here allow derived classes to set a default menu item
	ModifyMenu(submenu);

	/*
		There's a bug that requires a workaround here...
		Thanks to Paul DiLascia for pointing it out!
		Check MSDN for KnowledgeBase article Q135788 for more details.
	*/
	CPoint currloc;
	GetCursorPos(&currloc);
	m_ActivateWnd->SetForegroundWindow();
	submenu->TrackPopupMenu(0,currloc.x,currloc.y,m_ActivateWnd);
	m_ActivateWnd->PostMessage(WM_GETTEXTLENGTH,0,0);
}

void SystemTrayIcon::OnLButtonDblClick() {
	ASSERT(m_ActivateWnd);
	if (m_ActivateWnd) {

		CMenu menu;
		BOOL success;

		ASSERT(m_ActivateWnd);			// check window that will own menu
		if (!m_ActivateWnd) return;

		ASSERT(m_Menu);					// check menu identifier
		if (!m_Menu) return;

		success = menu.LoadMenu(m_Menu);// load the menu
		ASSERT(success);				// check success of menu load
		if (!success) return;

		CMenu* submenu = menu.GetSubMenu(0); // extract submenu
		ASSERT(submenu);				// check success of submenu
		if (!submenu) return;

		// here allow derived classes to set a default menu item
		ModifyMenu(submenu);

		// now the default is set, suck it out
		UINT item = ::GetMenuDefaultItem(submenu->m_hMenu,FALSE,0);
		if ( item == -1) {				// if no default, activate window
			m_ActivateWnd->ShowWindow(SW_NORMAL);
			m_ActivateWnd->SetForegroundWindow();
		}
		else {
			m_ActivateWnd->SendMessage(WM_COMMAND,item,0);
		}
	}
}

void SystemTrayIcon::ModifyMenu(CMenu* menu) {
	ASSERT(menu);
	if (!menu) return;
	ASSERT(menu->m_hMenu);
	if (!menu->m_hMenu) return;

	BOOL result = ::SetMenuDefaultItem(menu->m_hMenu,0,TRUE);
	ASSERT(result);
}

void SystemTrayIcon::internalSetIcon(HICON hicon) {
	if (!hicon || !m_Enabled) {
		Enable(FALSE);					// disabled or disabling
		m_Nid.hIcon = hicon;
	}
	else {								// enabled and valid new icon
		m_Nid.uFlags = NIF_ICON;
		m_Nid.hIcon = hicon;
		VERIFY(Shell_NotifyIcon(NIM_MODIFY,&m_Nid));
	}
}

LRESULT SystemTrayIcon::OnSystemTrayNotify(WPARAM id,LPARAM event) {
	ASSERT(id==(UINT)this);				// data unexpectedly corrupted
	LRESULT handled = 1;				// assume handled
	switch (event) {
	case WM_MOUSEMOVE:		OnMouseMove();			break;

	case WM_LBUTTONDBLCLK:	OnLButtonDblClick();	break;
	case WM_LBUTTONDOWN:	OnLButtonDown();		break;
	case WM_LBUTTONUP:		OnLButtonUp();			break;

	case WM_MBUTTONDBLCLK:	OnMButtonDblClick();	break;
	case WM_MBUTTONDOWN:	OnMButtonDown();		break;
	case WM_MBUTTONUP:		OnMButtonUp();			break;

	case WM_RBUTTONDBLCLK:	OnRButtonDblClick();	break;
	case WM_RBUTTONDOWN:	OnRButtonDown();		break;
	case WM_RBUTTONUP:		OnRButtonUp();			break;

	default:				ASSERT(0); handled = 0;	break;
	}

	return handled;
}

//	CWnd SetForegroundWindow TrackPopupWindow Shell_NotifyIcon
//		friend CMenu


/////////////////////////////////////////////////////////////////////////////
// CNotifyTargetWnd

IMPLEMENT_DYNAMIC(CNotifyTargetWnd, CWnd)

CNotifyTargetWnd::CNotifyTargetWnd()
{
}

CNotifyTargetWnd::~CNotifyTargetWnd()
{
}


BEGIN_MESSAGE_MAP(CNotifyTargetWnd, CWnd)
	ON_MESSAGE(WM_SYSTEM_TRAY_NOTIFY,OnSystemTrayNotify)
	//{{AFX_MSG_MAP(CNotifyTargetWnd)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CNotifyTargetWnd message handlers
LRESULT CNotifyTargetWnd::OnSystemTrayNotify(WPARAM id,LPARAM event) {
	ASSERT(m_Sti);
	if (m_Sti) return m_Sti->OnSystemTrayNotify(id,event);
	else return 0;						// didn't handle
}

void CNotifyTargetWnd::PostNcDestroy() 
{
	CWnd::PostNcDestroy();

	delete this;
}
