// TrayLaunch.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"

#include <objbase.h>

#include "TrayLnch.h"

#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static CDialog* in_about = NULL;

/////////////////////////////////////////////////////////////////////////////
// CTrayLaunchApp

BEGIN_MESSAGE_MAP(CTrayLaunchApp, CWinApp)
	//{{AFX_MSG_MAP(CTrayLaunchApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_UPDATE_COMMAND_UI(ID_APP_ABOUT, OnUpdateAppAbout)
	ON_UPDATE_COMMAND_UI(ID_APP_EXIT, OnUpdateAppExit)
	ON_UPDATE_COMMAND_UI(ID_APP_EXIT_ATTEMPT, OnUpdateAppExitAttempt)
	ON_COMMAND(ID_APP_EXIT_ATTEMPT, OnAppExitAttempt)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTrayLaunchApp construction

CTrayLaunchApp::CTrayLaunchApp()
: m_pmutex(NULL)
{
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CTrayLaunchApp object

CTrayLaunchApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CTrayLaunchApp initialization

BOOL CTrayLaunchApp::InitInstance()
{
	m_pmutex = new CMutex(FALSE,"ZiffDavisTrayLaunchAlreadyRunning");
	ASSERT(m_pmutex);
	if (!m_pmutex) {
#ifdef _DEBUG
		MessageBox(NULL,"Could not create mutex","Exiting",MB_OK);
#endif
		return FALSE;
	}
	if (!m_pmutex->Lock(0)) {
#ifdef _DEBUG
		MessageBox(NULL,"Could not lock mutex","Exiting",MB_OK);
#endif
		delete m_pmutex;
		m_pmutex=NULL;
		return FALSE;
	}
	CoInitialize(NULL);
	CMainFrame* pMainFrame = new CMainFrame;
	if (!pMainFrame->LoadFrame(IDR_MAINFRAME,WS_OVERLAPPEDWINDOW)) return FALSE;

	// save window handle
	m_pMainWnd = pMainFrame;
	m_pMainWnd->ShowWindow(SW_HIDE);
	m_pMainWnd->UpdateWindow();
	pMainFrame->SetupTrayIcon();

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	virtual BOOL OnInitDialog();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static int lets_get_outta_here = 0;

// App command to run the dialog
void CTrayLaunchApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	if (in_about==NULL) {
		in_about=&aboutDlg;
		aboutDlg.DoModal();
		in_about=NULL;
		if (lets_get_outta_here) { // we were just waiting for this to be gone
			lets_get_outta_here = 0;
			m_pMainWnd->PostMessage(WM_COMMAND,ID_APP_EXIT);
		}
	}
	else in_about->SetForegroundWindow();
}

void CTrayLaunchApp::OnUpdateAppExitAttempt(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);
}

void CTrayLaunchApp::OnAppExitAttempt() 
{
	if (in_about) {
		lets_get_outta_here = 1;
		in_about->EndDialog(IDOK);
	}
	else { // ready to go
		m_pMainWnd->PostMessage(WM_COMMAND,ID_APP_EXIT);
	}
}

void CTrayLaunchApp::OnUpdateAppAbout(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);
}

void CTrayLaunchApp::OnUpdateAppExit(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// CTrayLaunchApp commands

BOOL CTrayLaunchApp::PreTranslateMessage(MSG* pMsg)
{
	return CWinApp::PreTranslateMessage(pMsg);
}

int CTrayLaunchApp::ExitInstance() 
{
	CoUninitialize();
	if (m_pmutex) {
		m_pmutex->Unlock();
		delete m_pmutex;
		m_pmutex=NULL;
	}
	return CWinApp::ExitInstance();
}

BOOL CAboutDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	SetForegroundWindow();
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}
